import React, {Component} from 'react';
import OptionsService from '../services/network/fetchSelectOptions'
import PropTypes from "prop-types";
import Loading from "../Loading";
import {map} from 'lodash'
import {TableComponent, PaginationComponent} from '../common/components/index'


class TableView extends Component {
    static propTypes = {
        table: PropTypes.shape().isRequired,
        token: PropTypes.string.isRequired
    }

    constructor(props) {
        super(props);
        this.state = {
            query: this.props.table.query,
            result: 'onMount',
            size: 0,
            total: 0,
            loading: true,
            error: ({anErrorOccurred: false, message: null}),
            page: 0,
            lastWorkingQuery: this.props.table.query
        }
    }

    componentDidUpdate(prevProps, prevState) {
        if (this.state.query !== null && this.state.query.endpoint !== this.props.table.query.endpoint) {
            this.state.loading = false;
            this.setState({query: this.props.table.query, loading: true})
        }
        if (this.state.result === 'onMount') {
            this.state.loading = false;
            this.fetchTable(this.props.token, this.state.query)
        } else {
            if (this.state.loading) {
                this.state.loading = false;
                this.fetchTable(this.props.token, this.state.query)
            }
        }
    }

    componentDidMount() {
        this.handlerOnSearch({onMount: 'onMount'})
    }

    handlerOnSearch(query) {
        map(query, (value, key) => {
            this.setState(prevState => {
                let query = prevState.query
                switch (key) {
                    case 'default':
                        map(query.search[0], (value, key) => {
                            query.search[0][key] = null
                        })
                        break
                    case 'onMount':
                        query = this.state.query;
                        break
                    case 'start':
                        query.search[0][key] = value
                        this.setState({page: value, query: query, loading: true})
                        break
                    case 'reload':
                        query = this.state.lastWorkingQuery
                        break
                    default:
                        query.search[0][key] = value;
                        break
                }
                return ({
                    loading: true,
                    query: query
                })
            })
        })
    }

    fetchTable(token, query) {
        OptionsService.fetchTable(token, query)
            .then(obj => {
                if (obj.message) {
                    this.setState(() => {
                            let state = {
                                error: ({anErrorOccurred: true, message: obj.message}),
                                loading: false,
                                query: this.state.lastWorkingQuery,
                                result: false,
                                size: 1,
                                total: 1,
                                pageCount: 1
                            }
                            if (query === this.props.table.query) { //failed to fallback to default query
                                console.log(obj)
                                console.log('failed to set default query')
                                state.query = null
                            }
                            return state
                        }
                    )
                } else {
                    this.setState({
                        error: ({anErrorOccurred: false, message: null}),
                        loading: false,
                        result: obj.result,
                        size: obj.size,
                        total: obj.total,
                        pageCount: Math.trunc(obj.total / obj.size),
                        lastWorkingQuery: query
                    })
                }


                }
            )
    }

    render() {
        return (
            <div>
                {this.state.loading ? <Loading/> :
                    <div>
                        <TableComponent data={this.state.result}
                                        head={this.props.table.headers[0]}
                                        handleQuery={(e) => {
                                            this.handlerOnSearch(e)
                                        }}
                                        query={this.state.query}
                                        error={this.state.error}
                                        onUpdate={this.props.table.functions[0]}
                                        onDeactivate={this.props.table.functions[1]}
                                        styles={this.props.table.styles}
                        />
                        <PaginationComponent
                            handleQuery={(e) => {
                                this.handlerOnSearch(e)
                            }}
                            size={this.state.size}
                            total={this.state.total}
                            pageCount={this.state.pageCount}
                            page={this.state.page}
                        />
                    </div>
                }
            </div>
        )
    }
}

export default TableView;
